<?php
/**
Copyright Guillaume Bourgeois (2018)

contact@souke.fr

Ce logiciel est un programme informatique servant à aider les producteurs
à distribuer leur production en circuits courts.

Ce logiciel est régi par la licence CeCILL soumise au droit français et
respectant les principes de diffusion des logiciels libres. Vous pouvez
utiliser, modifier et/ou redistribuer ce programme sous les conditions
de la licence CeCILL telle que diffusée par le CEA, le CNRS et l'INRIA
sur le site "http://www.cecill.info".

En contrepartie de l'accessibilité au code source et des droits de copie,
de modification et de redistribution accordés par cette licence, il n'est
offert aux utilisateurs qu'une garantie limitée.  Pour les mêmes raisons,
seule une responsabilité restreinte pèse sur l'auteur du programme,  le
titulaire des droits patrimoniaux et les concédants successifs.

A cet égard  l'attention de l'utilisateur est attirée sur les risques
associés au chargement,  à l'utilisation,  à la modification et/ou au
développement et à la reproduction du logiciel par l'utilisateur étant
donné sa spécificité de logiciel libre, qui peut le rendre complexe à
manipuler et qui le réserve donc à des développeurs et des professionnels
avertis possédant  des  connaissances  informatiques approfondies.  Les
utilisateurs sont donc invités à charger  et  tester  l'adéquation  du
logiciel à leurs besoins dans des conditions permettant d'assurer la
sécurité de leurs systèmes et ou de leurs données et, plus généralement,
à l'utiliser et l'exploiter dans les mêmes conditions de sécurité.

Le fait que vous puissiez accéder à cet en-tête signifie que vous avez
pris connaissance de la licence CeCILL, et que vous en avez accepté les
termes.
 */

namespace domain\Document\Document;

use domain\Producer\Producer\ProducerSolver;
use domain\User\User\UserSolver;
use kartik\mpdf\Pdf;
use domain\_\AbstractManager;
use yii\base\ErrorException;

class DocumentManager extends AbstractManager
{
    protected UserSolver $userSolver;
    protected DocumentSolver $documentSolver;
    protected DocumentBuilder $documentBuilder;
    protected ProducerSolver $producerSolver;
    protected DocumentReferenceGenerator $documentReferenceGenerator;

    public function loadDependencies(): void
    {
        $this->userSolver = $this->loadService(UserSolver::class);
        $this->documentSolver = $this->loadService(DocumentSolver::class);
        $this->documentBuilder = $this->loadService(DocumentBuilder::class);
        $this->producerSolver = $this->loadService(ProducerSolver::class);
        $this->documentReferenceGenerator = $this->loadService(DocumentReferenceGenerator::class);
    }

    public function changeStatus(DocumentInterface $document, string $status): void
    {
        $document->status = $status;

        if ($status == Document::STATUS_VALID) {
            $this->documentReferenceGenerator->generateReference($document);
            $this->generatePdf($document, Pdf::DEST_FILE);
        }

        $this->documentBuilder->update($document);
    }

    public function validateDocument(Document $document): void
    {
        if($this->documentSolver->isStatusDraft($document)) {
            $this->changeStatus($document, Document::STATUS_VALID);
        }
    }

    public function generatePdf(DocumentInterface $document, string $destination): ?string
    {
        $producer = $document->producer;
        $content = \Yii::$app->controller->renderPartial('/document/download', [
            'producer' => $producer,
            'document' => $document
        ]);

        $contentFooter = '<div id="footer">';
        if ($this->producerSolver->getConfig('document_image_bottom')) {
            $urlDocumentImageBottom = \Yii::$app->urlManagerProducer->getHostInfo() . '/' . \Yii::$app->urlManagerProducer->baseUrl . '/uploads/' . $producer->document_image_bottom;
            $contentFooter .= '<div class="image"><img src="'.$urlDocumentImageBottom.'" style="max-height:80px;" /></div>';
        }
        if ($this->producerSolver->getConfig('document_infos_bottom')) {
            $contentFooter .= '<div class="infos-bottom">'.nl2br($producer->document_infos_bottom).'</div>';
        }
        $contentFooter .= '</div>';

        $marginBottom = 10;
        if (strlen($this->producerSolver->getConfig('document_infos_bottom')) > 0) {
            $marginBottom = 40;
        }

        $this->initDirectoryPdf($document);

        $pdf = new Pdf([
            'mode' => Pdf::MODE_UTF8,
            'format' => Pdf::FORMAT_A4,
            'orientation' => Pdf::ORIENT_PORTRAIT,
            'destination' => $destination,
            'content' => $content,
            'filename' => $this->documentSolver->getFilenameComplete($document),
            'cssFile' => \Yii::getAlias('@webroot/css/document/download.css'),
            'marginBottom' => $marginBottom,
            'methods' => [
                'SetHTMLFooter' => $contentFooter
            ]
        ]);

        return $pdf->render();
    }

    public function initDirectoryPdf(DocumentInterface $document): void
    {
        $aliasDirectoryBase = $this->documentSolver->getAliasDirectoryBase($document);
        $directoryPdf = \Yii::getAlias($aliasDirectoryBase);
        if (!file_exists($directoryPdf)) {
            mkdir($directoryPdf, 0755);
        }
    }

    public function downloadPdf(DocumentInterface $document, bool $regenerate = false)
    {
        $filenameComplete = $this->documentSolver->getFilenameComplete($document);

        if (!file_exists($filenameComplete) || $this->documentSolver->isStatusDraft($document) || $regenerate) {
            $this->generatePdf($document, Pdf::DEST_FILE);
        }

        if (file_exists($filenameComplete)) {
            return \Yii::$app->response->sendFile($filenameComplete, $this->documentSolver->getFilename($document), ['inline' => true]);
        } else {
            throw new ErrorException('File ' . $filenameComplete . ' not found');
        }
    }

    public function sendDocument(DocumentInterface $document): bool
    {
        if (isset($document->user) && strlen($document->user->email) > 0) {
            $producer = $this->getProducerContext();

            $subjectEmail = $this->documentSolver->getType($document);
            if ($this->documentSolver->isStatusValid($document)) {
                $subjectEmail .= ' N°' . $document->reference;
            }

            $email = \Yii::$app->mailerService->getMailer()->compose(
                [
                    'html' => '@common/mail/sendDocument-html',
                    'text' => '@common/mail/sendDocument-text'
                ], [
                'document' => $document,
                'producer' => $producer
            ])
                ->setTo($this->userSolver->getEmailSendingInvoicingDocuments($document->user))
                ->setFrom([$this->producerSolver->getProducerEmailPlatform($producer) => $producer->name])
                ->setSubject('[' . $producer->name . '] ' . $subjectEmail);

            $this->generatePdf($document, Pdf::DEST_FILE);
            $email->attach($this->documentSolver->getFilenameComplete($document));

            // @TODO : gérer via un événement
            $this->documentBuilder->updateDocumentIsSend($document, true);

            return $email->send();
        }

        return false;
    }
}